from django.core.management.base import  BaseCommand
from historylink.manager import manager
from django.contrib.contenttypes.models import ContentType




class Command(BaseCommand):
    
    def handle(self, *args, **options):
        
        if len(args) > 0:
            
            for arg in args:
                model = ContentType.objects.get_by_natural_key(*arg.split('.')).model_class()
                manager.syncModel(model)
                
        else:
            manager.syncAll()
        


from django.db.models.signals import post_save, post_delete
from django.contrib.contenttypes.models import ContentType

from historylink.models import HistoryLink

class AlreadyRegistered(Exception):
    pass


class ModelManager(object):
    
    def post_save(self, sender, instance, created, **kwargs):
        self.createHistoryLink(instance)
        
    def post_delete(self, sender, instance, **kwargs):
        HistoryLink.objects.filter(content_type=ContentType.objects.get_for_model(instance), object_id=instance.pk).delete()

    
    def createHistoryLink(self, o):
        HistoryLink.objects.get_or_create(content_type=ContentType.objects.get_for_model(o), 
                                                  object_id=o.pk, url=o.get_absolute_url(), 
                                                  defaults={'content_object' : o, 
                                                            'url' : o.get_absolute_url()})
    

class Manager(object):
    
    _registry = {}
    
    def register(self, models, manager=ModelManager):
        
        if not isinstance(models, (list, tuple,)):
            models = [models]
        
        for model in models:
            
            if model in self._registry:
                continue
            
            self._registry[model] = manager()
            
            post_save.connect(self._registry[model].post_save, sender=model)
            post_delete.connect(self._registry[model].post_delete, sender=model)
            
    def syncModel(self, model):
        
        ct = ContentType.objects.get_for_model(model)
        manager = self._registry.get(model)
        query = model.objects.all()
            
        print "syncing %s.%s (%s objects)" % (ct.app_label, ct.name, query.count())
        
        for o in query:
            manager.createHistoryLink(o)
        
    def syncAll(self):
        
        for model, manager in self._registry.iteritems():
            
            ct = ContentType.objects.get_for_model(model)
            
            query = model.objects.all()
            
            print "syncing %s.%s (%s objects)" % (ct.app_label, ct.name, query.count())
            
            for o in query:
                manager.createHistoryLink(o)
    
manager = Manager()


from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic

class HistoryLink(models.Model):
    
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    
    url = models.CharField(max_length=512)


#!/usr/bin/env python
from setuptools import setup, find_packages

import historylink

CLASSIFIERS = [
    'Intended Audience :: Developers',    
    'Natural Language :: English',
    'Operating System :: OS Independent',
    'Programming Language :: Python'    
]

KEYWORDS = 'django url change redirect 404 301'


setup(name = 'historylink',
    version = historylink.__version__,
    description = """Store model url change, od a permanent redirect on 404""",
    author = historylink.__author__,
    url = "https://github.com/oxys-net/django-historylink",
    packages = find_packages(),
    classifiers = CLASSIFIERS,
    keywords = KEYWORDS,
    zip_safe = True
)


from django.http import Http404, HttpResponsePermanentRedirect


from historylink.models import HistoryLink as HistoryLinkModel

class HistoryLink(object):
    """
    def process_exception(self, request, exception):
        
        if isinstance(exception, Http404):
            import pdb; pdb.set_trace()
            try:
                hl = HistoryLinkModel.objects.get(url=request.META.get('PATH_INFO'))
                
                if request.META.get('PATH_INFO') != hl.content_object.get_absolute_url():                
                    return HttpResponsePermanentRedirect(redirect_to=hl.content_object.get_absolute_url())
            except:
                return
            
    """        
            
    def process_response(self, request, response):
        if response.status_code != 404:
            return response # No need to check for a redirect for non-404 responses.
        path_qs = request.get_full_path().split('?')
        
        path = path_qs[0]
        if len(path_qs) > 1:
            qs = "?" + '?'.join(path_qs[1:])
        else:
            qs = ''
            
        try:
            hl = HistoryLinkModel.objects.get(url=path)
            
            if path != hl.content_object.get_absolute_url():                
                return HttpResponsePermanentRedirect(redirect_to="%s%s" %( hl.content_object.get_absolute_url(), qs))
        except:
            pass

        # No redirect was found. Return the response.
        return response
